//==============================================================================================
// Vehicle Monitor System
// JuLY 02, 2015
// Jim Abraham
// Arduino MEGA 2560 R3, uLCD-43PT, OBD-II UART
//==============================================================================================

#include <EEPROM.h>
#include <genieArduino.h>
Genie genie;

//-------------------------------------------------------------------------------
//Set up ring buffer
//-------------------------------------------------------------------------------
char rxData[20];
char rxIndex = 0;

//-------------------------------------------------------------------------------
// Vehicle Data Variables
//-------------------------------------------------------------------------------
int mph,rpm,VSS,MAF,MAF1,mpg_cnt=0,mpg_cnt1=0;
float gals_per_hour1=0,gals_per_hour2=0,gals_per_hour=0;
int gals_per_hour_int=0;
float Avg_miles_per_gal=0,Avg_miles_per_gal1=0,Avg_miles_per_gal2=0;
float Avg_miles_per_gal3=0,Avg_miles_per_gal4=0;
int Avg_miles_per_gal_int=0;
int fuel_level,fuel_level_prev,fuel_level_prev1,fl=0;
int fuel_level1,fuel_start=0;
int water_temp,air_temp;
int engine_load, throttle_position;
int odometer_reading;
char battery_voltage[4];
char bvIndex = 0;
int car_voltage;
char gals_left[4];
char glIndex = 0;
char obd_trouble_codes[20];//=("43 14 57 00 00 00 00");
char tcIndex = 0;
int obd_tc1,obd_tc2,obd_tc3,obd_tc4,obd_tc5,obd_tc6;
float miles_per_gal=0,miles_per_gal1=0,miles_per_gal2=0;
int miles_per_gal_int,miles_per_gal_int1;
int odometer_reading_strt=0;

//-------------------------------------------------------------------------------
int updateled0=0;
int updateled=0;
int update=10;

// the current address in the EEPROM (i.e. which byte we're going to write to next)
int addr = 0;

//=======================================================================================
// SET-UP
//=======================================================================================
void setup()
{
  Serial.begin(9600);      // prints to serial monitor

  Serial1.begin(115200);  // #1 LCD Communications.
  genie.Begin(Serial1);   // Use Serial1 for talking to the Genie Library, and to the 4D Systems display
  genie.AttachEventHandler(myGenieEventHandler); // Attach the user function Event Handler for processing events
//  genie.WriteObject(GENIE_OBJ_FORM,0,1); // change startup screen (#,1 #=form#)
  delay(1000);
  
// get mpg and display it.
  Mega_EEPROM_Read_MPG();

//Hardware serial connection to the obdii uart
  Serial3.begin(9600); 
 
//initiates obd link
  OBD_init(); 
}

//=======================================================================================
// MAIN PROGRAM
//=======================================================================================
void loop()
{
//-------------------------------------------------------------------------------
// Delete any data that may be left over in the serial port.
//-------------------------------------------------------------------------------
  Serial3.flush();  
  delay(500);

//-------------------------------------------------------------------------------
// Get Vehicle Data
//-------------------------------------------------------------------------------
  getMAF();
  getSPEED();
  calcMPG();
  getRPM();
  getENGINE_LOAD();
  getTHROTTLE_POSITION();
  getFUEL();
  getODOMETER_READING();

  update=update + 1;  // update every 10 seconds
  if (update > 10)
  {
    getVOLT();
    getWATER_TEMP();
    getINTAKE_AIR_TEMP();
    update=0;
  }
//-------------------------------------------------------------------------------
// Update LCD Forms
//-------------------------------------------------------------------------------
  Update_FORM0();
  Update_FORM1();
  Update_FORM3();
  Update_FORM4();
  Update_FORM6();
  
//-------------------------------------------------------------------------------
// This calls the library each loop to process the queued responses from the display  
//-------------------------------------------------------------------------------
  genie.DoEvents(); 
  
//-------------------------------------------------------------------------------
  delay(500);
}

//=======================================================================================
// ROUTINES
//=======================================================================================
// Write Data to Mega 2560 EEPROM
//-------------------------------------------------------------------------------
void Mega_EEPROM_Write_MPG()
{
  // need to divide by 4 because analog inputs range from
  // 0 to 1023 and each byte of the EEPROM can only hold a value from 0 to 255.
  // 30.4 mpg 
  // val =30
  // 30.4 x 10 =304, 30 x 10 =300 so 304-300 = 4.
  // val1=4
  int val = miles_per_gal;
  int val1 = miles_per_gal*10 - val*10;
  int val2 = gals_per_hour;
  int val3 = gals_per_hour*10 - val2*10;
  // write the value to the appropriate byte of the EEPROM.
  // these values will remain there when the board is turned off.
  EEPROM.write(0, val);
  EEPROM.write(1, val1);
  delay(100);
  EEPROM.write(2, val2);
  EEPROM.write(3, val3);
  delay(100);
}

void Mega_EEPROM_Write_Avg_MPG()
{
  int val4 = Avg_miles_per_gal;
  int val5 = Avg_miles_per_gal*10 - val4*10;
  EEPROM.write(4, val4);
  EEPROM.write(5, val5);
  delay(100);
}

//-------------------------------------------------------------------------------
// Read Data to Mega 2560 EEPROM
//-------------------------------------------------------------------------------
void Mega_EEPROM_Read_MPG()
//-------------------------------------------------------------------------------
{
  miles_per_gal_int = EEPROM.read(0)*10 + EEPROM.read(1);
  miles_per_gal=miles_per_gal_int/10;
  
  gals_per_hour_int = EEPROM.read(2)*10 + EEPROM.read(3);
  gals_per_hour=gals_per_hour_int/10;

  Avg_miles_per_gal_int = EEPROM.read(4)*10 + EEPROM.read(5);
  Avg_miles_per_gal=Avg_miles_per_gal_int/10;
  Avg_miles_per_gal4 = Avg_miles_per_gal;
  delay(500);  
  genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x06, miles_per_gal_int);
  genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x05, gals_per_hour_int);
  genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x16, Avg_miles_per_gal_int); 
}

//-------------------------------------------------------------------------------
// INITIALIZE OBD-II UART
//-------------------------------------------------------------------------------
void OBD_init(void)
{
  //Wait for a little while before sending the reset command to the OBD-II-UART
  delay(1000);
  //Reset the OBD-II-UART
  Serial3.println("ATZ");
  OBD_read();
  delay(250);   //Wait for a bit before starting to send commands after the reset
  Serial3.println("ATSP0");//configured to the correct OBD protocol
  OBD_read();
  delay(250);
  Serial3.println("ATE0"); //turning off character echoing
  OBD_read();
  delay(250);
  Serial3.flush();
}

//-------------------------------------------------------------------------------
// GET SPEED (0-158 mph)
//-------------------------------------------------------------------------------
void getMAF()
{
//PID
//(hex)	Data bytes returned	Description	    Min value	Max value   Units	Formula
//10	2	                MAF air flow rate	0	655.35	    grams/sec	((A*256)+B) / 100
//Query the OBD-II-UART for the MAF air flow rate
  Serial3.flush();
  Serial3.println("0110");
  OBD_read();
  MAF1 = ((strtol(&rxData[6], 0, 16) * 256) + strtol(&rxData[9], 0, 16)) / 100;  
  if (MAF1>0){MAF=MAF1;}
}
//-------------------------------------------------------------------------------
// GET SPEED (0-158 mph)
//-------------------------------------------------------------------------------
void getSPEED()
{
  //Query the OBD-II-UART for the vehicle speed
  Serial3.flush();
  Serial3.println("010D");
  OBD_read();
  mph = (int)((strtol(&rxData[6], 0, 16) * 10000L + 5) / 16090); //convert kmh to mph
  VSS = strtol(&rxData[6], 0, 16); //kmh
}

//-------------------------------------------------------------------------------
// GET RPM (0-16,383.75 rpm)
//-------------------------------------------------------------------------------
void getRPM()
{
  //Query the OBD-II-UART for the Vehicle rpm
  Serial3.flush();
  Serial3.println("010C");
  OBD_read();
  rpm=((strtol(&rxData[6], 0, 16) * 256) + strtol(&rxData[9], 0, 16)) / 4;
}

//-------------------------------------------------------------------------------
// GET WATER TEMPERATURE (-40 TO 419 DegF)
//-------------------------------------------------------------------------------
void getWATER_TEMP()
{
  //Query the OBD-II-UART for the Engine Coolant Temp
  Serial3.flush();
  Serial3.println("0105");
  OBD_read();
  water_temp= (1.8 * (strtol(&rxData[6], 0, 16) - 40) + 32)*10;
}

//-------------------------------------------------------------------------------
// GET AIR TEMPERATURE (-40 TO 419 DegF)
//-------------------------------------------------------------------------------
void getINTAKE_AIR_TEMP()
{
  //Query the OBD-II-UART for the Engine AIR Temp
  Serial3.flush();
  Serial3.println("010F");
  OBD_read();
  air_temp= (1.8 * (strtol(&rxData[6], 0, 16) - 40) + 32)*10;
}

//-------------------------------------------------------------------------------
// GET ENGINE LOAD (0-100%)
//-------------------------------------------------------------------------------
void getENGINE_LOAD()
{
  //Query the OBD-II-UART for the Engine Load
  Serial3.flush();
  Serial3.println("0104");
  OBD_read();
  engine_load= (strtol(&rxData[6], 0, 16)*100)/255;
}

//-------------------------------------------------------------------------------
// GET THROTTLE POSITION (0-100%)
//-------------------------------------------------------------------------------
void getTHROTTLE_POSITION()
{
  //Query the OBD-II-UART for the THROTTLE POSITION
  Serial3.flush();
  Serial3.println("0111");
  OBD_read();
  throttle_position= (strtol(&rxData[6], 0, 16)*100)/255;
}

//-------------------------------------------------------------------------------
// GET FUEL LEVEL (0-100%)
//-------------------------------------------------------------------------------
void getFUEL()
{//start up delay
  //Query the OBD-II-UART for the fuel
  Serial3.flush();
  Serial3.println("012F");
  OBD_read();
  // = 76 * 1.32 = 10.0 gals in tank, send 100 to lcd for 10.0
  fuel_level= ((((strtol(&rxData[6], 0, 16)* 100)/255)) * 1.32) + 10; //int 
}
//-------------------------------------------------------------------------------
// GET ODOMETER READING
// odometer reading = Distance traveled since codes cleared.
// Will go to zero when trouble code reset.
//-------------------------------------------------------------------------------
void getODOMETER_READING()
{
  //Query the OBD-II-UART for the Distance Traveled
  Serial3.flush();
  Serial3.println("0131");
  OBD_read();
  if ((mph != 6) || (mph==0)) 
    {
      odometer_reading= 0.62137 * (((strtol(&rxData[6], 0, 16) * 256) + strtol(&rxData[9], 0, 16)));
    }
  // km = (A*256)+B
  // 1 kilometer = 0.621371192 miles
  // 1 mile = 1.609344 kilometers
}

//-------------------------------------------------------------------------------
// CALCULATE MILES PER GALLON
//-------------------------------------------------------------------------------
void calcMPG()
{
/*
 OBD-II VSS reading is in kilometers/hour and MAF reading is grams/sec times 100.
 Convert the MAF to Gallons of fuel per hour (GPH) and then calculated miles per gallon (MPG). 
 Most vehicles do not have a fuel flow sensor, therfore use the Mass Air Flow (MAF) sensor 
 and the Vehicle Speed Sensor (VSS) to calculate miles per gallon (MPG). 
 1. Mass Air Flow - The mass of Air in grams per second consumed.
 2. Vehicle Speed Sensor - The actual speed of the vehicle. 
 Modern vehicles use the oxygen sensors to feedback data to the vehicle's electronic control 
 module (ECM) and control the air to fuel ratio. 
 This ratio is set at the chemically ideal value of 14.7 grams of air to every gram of gasoline.
 The Mass Air Flow is the mass of air consumed in grams per second. 
 Used this information to convert the Mass Air Flow to gallons per hour and then calculate miles per gallon. 

 Here are the steps to make the conversion:
 1.Divide the MAF by 14.7 to get grams of fuel per second 
 2.Divide result by 454 to get Lbs fuel per second 
 3.Divide result by 6.17 Gals fuel per second 
 4.Multiply result by 3600 to get gallons per hour 
 The expression for GPH is: MAF * 0.0805. 
 The value for vehicle speed is delivered in Km/Hr, to convert to miles multiply by
 0.621317. To calculate MPG divide the MPH by GPH. The final math expression for MPG will be:
 VSS * 7.718 / MAF
((MAF /14.7)/454)/6.721) x 3600
  Conversion Table
   454 grams 1 LB
   6.17 Lbs Gas 1 Gal
   3600 sec 1 Hour
   0.621317 Mile 1 Km
*///-------------------------------------------------------------------------------

if (mph != 6 && mph>0)
 {
//----------------------------------------------------------------
//GPH = (14.7 * 6.17 * 454) / (3600 * MAF / 100)
//    = 1143.8 / MAF   
//----------------------------------------------------------------
   gals_per_hour1=  MAF * 0.0805;
//----------------------------------------------------------------
//MPG = (14.7 * 6.17 * 454 * VSS * 0.621371) / (3600 * MAF / 100)
//    = 710.7 * VSS / MAF  
//----------------------------------------------------------------
   miles_per_gal1= (710.7 * VSS) / MAF;
//----------------------------------------------------------------
  mpg_cnt++;
  gals_per_hour2 = gals_per_hour2 + gals_per_hour1;

  if (miles_per_gal1 > 4100) {miles_per_gal1=4100;}
  miles_per_gal2 = miles_per_gal2 + miles_per_gal1;

  if (mpg_cnt >=30) //30 sec avg
   {
     gals_per_hour = gals_per_hour2/mpg_cnt;
     gals_per_hour_int = gals_per_hour*10;

     miles_per_gal = miles_per_gal2/mpg_cnt;
     miles_per_gal_int = miles_per_gal/10;
     Avg_miles_per_gal1 = Avg_miles_per_gal1 + miles_per_gal;

     Mega_EEPROM_Write_MPG();
     gals_per_hour2=0;
     miles_per_gal2=0;
     mpg_cnt=0;
     mpg_cnt1++;
   }
  if (mpg_cnt1 >=20) //10 min avg
    {//10 min avg
       Avg_miles_per_gal2 = Avg_miles_per_gal1/mpg_cnt1;
       Avg_miles_per_gal3= (0.8 * Avg_miles_per_gal2)+(0.2 * Avg_miles_per_gal4);
       Avg_miles_per_gal = Avg_miles_per_gal3;
       Avg_miles_per_gal_int = Avg_miles_per_gal/10;
       Mega_EEPROM_Write_Avg_MPG();
       Avg_miles_per_gal4 = Avg_miles_per_gal;
       Avg_miles_per_gal1=0;
       mpg_cnt1=0;
    }
  }
}

//-------------------------------------------------------------------------------
// GET TROUBLE CODES 
//-------------------------------------------------------------------------------
void getTROUBLE_CODES()
{
  //Query the OBD-II-UART for the Stored TROUBLE Codes
  Serial3.flush();
  Serial3.println("03");
  obd_tc_read();
  //43 14 57 00 00 00 00
  //remove 43 so end with sp14 57 00 00 00 00
  //                       012345678901234567 
  obd_tc1=(strtol(&obd_trouble_codes[0], 0, 16));   //14h =20d
  obd_tc2=(strtol(&obd_trouble_codes[3], 0, 16));   //57 = 87d
  obd_tc3=(strtol(&obd_trouble_codes[6], 0, 16));   //00
  obd_tc4=(strtol(&obd_trouble_codes[9], 0, 16));   //00
  obd_tc5=(strtol(&obd_trouble_codes[12], 0, 16));  //00
  obd_tc6=(strtol(&obd_trouble_codes[15], 0, 16));  //00
  Update_FORM3_TC;
}
//-------------------------------------------------------------------------------
// Read Stored TROUBLE Codes
  //43 14 57 00 00 00 00
  //remove 43 so end with sp14 57 00 00 00 00
  //                       012345678901234567 
//-------------------------------------------------------------------------------
void obd_tc_read(void)
{
  char c;
  int fc4=0;
  
  do {
    if (Serial3.available() > 0)
    {
     c = Serial3.read();
     if ((c == '4') && (fc4 == 0)){fc4=1;}
     
     if ((c == '3') && (fc4 == 1)){fc4=2;}  
     
     if ((fc4 == 3) && (c != '>') && (c != '\r') && (c != '\n'))
      { 
        obd_trouble_codes[tcIndex++] =  c;  
      }
    if (fc4 == 2){fc4=3;} 
    }
  } while (c != '>'); 
  obd_trouble_codes[tcIndex++] = '\0';
  tcIndex = 0;
}
//-------------------------------------------------------------------------------
// RESET TROUBLE CODES
//-------------------------------------------------------------------------------
void reset_TROUBLE_CODES()
{
  //Send code to OBD-II-UART to reset trouble codes
  Serial3.flush();
  Serial3.println("04");
  OBD_read();
}
//-------------------------------------------------------------------------------
// READ CAR STATUS
//-------------------------------------------------------------------------------
void OBD_read(void)
{
  char c;
  do {
    if (Serial3.available() > 0)
    {
      c = Serial3.read();
      if ((c != '>') && (c != '\r') && (c != '\n')) //Keep these out of our buffer
      {
        rxData[rxIndex++] = c; //Add whatever we receive to the buffer
      }
    }
  } while (c != '>'); //The ELM327 ends its response with this char so when we get it we exit out.
  rxData[rxIndex++] = '\0';//Converts the array into a string
  rxIndex = 0; //Set this to 0 so next time we call the read we get a "clean buffer"
} 
//strtol long int strtol (const char* str, char** endptr, int base);Convert string to long integer
//Parses the C-string str interpreting its content as an integral number of the specified base, 
//which is returned as a long int value. If endptr is not a null pointer, the function also sets 
//the value of endptr to point to the first character after the number.

//The function first discards as many whitespace characters as necessary until the first 
//non-whitespace character is found. Then, starting from this character, takes as many characters 
//as possible that are valid following a syntax that depends on the base parameter, and interprets 
//them as a numerical value. Finally, a pointer to the first character following the integer representation
//in str is stored in the object pointed by endptr.

//-------------------------------------------------------------------------------
// GET BATTERY VOLTAGE (0 TO 65 VDC)
//-------------------------------------------------------------------------------
void getVOLT()
{
  //Query the OBD-II-UART for the battery voltage
  Serial3.flush();
  Serial3.println("atrv");
  OBD_VOLT_read();
  car_voltage=(strtol(&battery_voltage[0], 0, 10));
//  battery_voltage= strtol(&rxData[6], 0, 16);  
  // = ((A*256)+B)/1000
//  Serial.print ("battery_voltage=");
//  Serial.print (rxData[0]);   =1
//  Serial.print (rxData[1]);   =2
//  Serial.print (rxData[2]);   =.
//  Serial.println (rxData[3]); =4
//  battery_voltage = str(rxData[0] & rxData[1] & rxData[2] & rxData[3]);
}
//-------------------------------------------------------------------------------
// Read Car Battery voltage using "atrv" command.
// on LCD Leddigits6 format ##.#
//-------------------------------------------------------------------------------
void OBD_VOLT_read(void)
{
  char c;
  do {
    if (Serial3.available() > 0)
    {
      c = Serial3.read();
      if ((c != '>') && (c != '\r') && (c != '\n') && (c != 'V')&& (c != '.')) //Keep these out of our buffer
      {
         battery_voltage[bvIndex++] =  c;  
      }
    }
  } while (c != '>'); //The ELM327 ends its response with this char so when we get it we exit out.
  battery_voltage[bvIndex++] = '\0';//Converts the array into a string
  bvIndex = 0; //Set this to 0 so next time we call the read we get a "clean buffer" 
}

//=======================================================================================
// Update Form 0
//=======================================================================================
void Update_FORM0()
{
  updateled= updateled+1;
  if (updateled==1){updateled0=1;}
  if (updateled==2)
    {  
      updateled0=0;
      updateled=0;
    }
  if (mph<200){genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x00, mph);}   //speed
  if ((mph>1) && (mph!=6) && (rpm!=640))
    {
      if (mph<200){genie.WriteObject(GENIE_OBJ_SCOPE, 0x00, mph);}    //speed chart (0-100)
      genie.WriteObject(GENIE_OBJ_SCOPE, 0x00, rpm/80);//rpm chart (0-8000) MAKE FIT IN 0-100 CHART
//      genie.WriteObject(GENIE_OBJ_SCOPE, 0x00, water_temp * 0.025);//water temp chart (0-400)MAKE FIT IN 0-100 CHART
    }  
  genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x01, rpm);  
  
  if (update==0)
  // bottom values = mpg   gph   bat volt
  {
    genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x06, miles_per_gal_int);
    genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x05, gals_per_hour_int);
    genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x04, car_voltage);
  }
  genie.WriteObject(GENIE_OBJ_USER_LED, 0x00, updateled0); //wdt led  
}
//=======================================================================================
// Update Form 1
//=======================================================================================
void Update_FORM1()
{
  if (mph<200){genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x0B, mph);}   //speed
  genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x0C, rpm);   //rpm
  if (update==0)
    {
      genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x02, throttle_position);
      genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x0A, engine_load);
      if ((air_temp < 3000) && (air_temp > 0)) {genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x09, air_temp);}
      if ((water_temp < 4000) && (water_temp > 0)) {genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x08, water_temp);}
      genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x07, fuel_level);  
      genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x03, car_voltage);
    } 
}
//=======================================================================================
// Update Form 2
//=======================================================================================
//void Update_FORM2()
//{
//  genie.WriteObject(GENIE_OBJ_COOL_GAUGE, 0x00, mph);   
//  genie.WriteObject(GENIE_OBJ_TANK, 0x01, rpm/10);
//  genie.WriteObject(GENIE_OBJ_METER, 0x00, water_temp/10);
//  genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x0D, odometer_reading);
//  genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x14, miles_per_gal*10); 
//}

//=======================================================================================
// Update Form 3
//=======================================================================================
void Update_FORM3()
{
// DISPLAY MILAGE INFO 
   genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x15, odometer_reading);
   genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x16, Avg_miles_per_gal_int); 
}
//=======================================================================================
// Update Form 3 TROUBLE CODES 
//=======================================================================================
void Update_FORM3_TC()
{
// DISPLAY TROUBLE CODES 
   genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x0E, obd_tc1);
   genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x0F, obd_tc2);
   genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x10, obd_tc3);
   genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x11, obd_tc4);
   genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x12, obd_tc5);
   genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x13, obd_tc6);
}
//=======================================================================================
// Update Form 4
//=======================================================================================
void Update_FORM4()
{
//  genie.WriteObject(GENIE_OBJ_COOL_GAUGE, 0x03, water_temp/10);   
  if ((water_temp < 4000) && (water_temp > 0)) 
    {
      genie.WriteObject(GENIE_OBJ_SCOPE, 0x01, water_temp/30);
      genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x0D, water_temp); 
      genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x18, air_temp); 
    }
}
//=======================================================================================
// Update Form 6
//=======================================================================================
void Update_FORM6()
{
//  genie.WriteObject(GENIE_OBJ_COOL_GAUGE, 0x03, water_temp/10);   
  if (mph != 6) 
    {
      genie.WriteObject(GENIE_OBJ_SCOPE, 0x02, engine_load*1.5);
      genie.WriteObject(GENIE_OBJ_SCOPE, 0x02, throttle_position*1.5);
      genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x17, engine_load); 
      genie.WriteObject(GENIE_OBJ_LED_DIGITS, 0x14, throttle_position); 
    }
}
//=======================================================================================
// This is the user's event handler. It is called by genieDoEvents()
// when the following conditions are true
// The link is in an IDLE state, and there is an event to handle
//=======================================================================================
void myGenieEventHandler(void)
{
  genieFrame Event;
  genie.DequeueEvent(&Event);

  //If the cmd received is from a Reported Event (Events triggered from the Events tab of Workshop4 objects)
  if (Event.reportObject.cmd == GENIE_REPORT_EVENT)
  {
    if (Event.reportObject.object == GENIE_OBJ_4DBUTTON)                // If the Reported Message was from a Slider
    {
      //------------------------------------------------------------------
      // Display TROUBLE Codes PB on Form 3
      //------------------------------------------------------------------
      if (Event.reportObject.index == 2)                                // If Display TROUBLE Codes PB on Form 3.
      {
        getTROUBLE_CODES();
      }
      //------------------------------------------------------------------
      // Reset TROUBLE Codes PB on Form 3
      //------------------------------------------------------------------
      if (Event.reportObject.index == 0)                                // If Reset TROUBLE Codes PB on Form 3.
      {
         reset_TROUBLE_CODES();
      }
    }
  }
}

